import os
import sqlite3
import logging
import traceback
import random 
import re

class EventDb(object):
    def __init__(self, name=None, persist=False):
        if name:
            self.db_name = name
        else:
            self.db_name = self.get_default_db()
        self.persist = persist
        self.set_logger()
        conn = sqlite3.connect(self.db_name)
        cursor = conn.cursor()
        cursor.execute('''
          CREATE TABLE IF NOT EXISTS events
          ([id] INTEGER PRIMARY KEY,
          [datetime] TEXT,
          [command] TEXT,
          [pid] INTEGER,
          [uid] INTEGER,
          [event] TEXT,
          [result] INTEGER,
          [detail] TEXT)
          ''')
        conn.commit()
        conn.close()

    def get_default_db(self):
        r = random.randint(0, 1000000) 
        return f"/var/tmp/{r}.event.db"

    def set_logger(self):
        self.logger_name = 'pswatch'
        self.logger = logging.getLogger(self.logger_name)

    def count(self):
        conn = sqlite3.connect(self.db_name)
        cur = conn.cursor()
        cur.execute('SELECT COUNT(*) from events')
        result = cur.fetchone()
        cur.close()
        conn.close()
        return result[0]

    def event_to_tuple(self, event):
        result = 1 if event['ok'] else 0
        return (event['datetime'], event['comm'], str(event['pid']), str(event['uid']), event['name'], result, event['detail'])

    def event_to_dict(self, evt):
        id,datetime,command,pid,uid,name,result,detail = evt
        event = {}
        ok = result > 0
        event['id'] = id
        event['datetime'] = datetime
        event['comm'] = command
        event['pid'] = pid
        event['uid'] = uid
        event['name'] = name
        event['ok'] = ok
        event['detail'] = detail
        return event

    def read(self, row_start, count):
        conn = sqlite3.connect(self.db_name)
        cur = conn.cursor()
        row_id_start = row_start + 1
        row_id_end = row_id_start + count
        result = []
        sql = f"SELECT id,datetime, command, pid, uid, event, result, detail FROM events WHERE id >={row_id_start} and id < {row_id_end}"
        for row in cur.execute(sql):
            event = self.event_to_dict(row)
            result.append(event)
        cur.close()
        conn.close()
        return result

    def write(self, events):
        conn = sqlite3.connect(self.db_name)
        cur = conn.cursor()
        event_tuples = []
        for evt in events:
            t = self.event_to_tuple(evt)
            event_tuples.append(t)
        sql = "INSERT INTO events(datetime, command, pid, uid, event, result, detail) VALUES(?, ?, ?, ?, ?, ?, ?)"
        cur.executemany(sql, event_tuples)
        conn.commit()
        cur.close()
        conn.close()

    def clear(self):
        conn = sqlite3.connect(self.db_name)
        cur = conn.cursor()
        sql = "DELETE FROM events"
        cur.execute(sql)
        conn.commit()
        cur.close()
        conn.close()

    def filter_check(self, event, filter_text):
        filter_out = True
        if filter_text == None:
            return False
        for evt_field in event:
            value = str(evt_field)
            if re.search(filter_text, value):
                filter_out = False
                break
        return filter_out

    def filter(self, filter_text):
        result = []
        conn = sqlite3.connect(self.db_name)
        cur = conn.cursor()
        sql = "SELECT id,datetime, command, pid, uid, event, result, detail FROM events"
        for row in cur.execute(sql):
            filter_out = self.filter_check(row, filter_text)
            if filter_out:
                continue
            event = self.event_to_dict(row)
            result.append(event)
        cur.close()
        conn.close()
        return result

    def __del__(self):
        if not self.persist and os.path.exists(self.db_name):
            os.unlink(self.db_name)
