#!/usr/bin/python3
# vim: set filetype=python
import os
import sys
import re
import json
import subprocess
import time
from .tracepoint import TracePoint
from .event_file import EventFileOpen,EventFileUnlink,EventFileTruncate,\
     EventFileChown,EventFileChmod,EventFileRmdir,EventFileMkdir,EventFileSymlink
from .event_socket import EventSocketListen,EventSocketConnect,EventSocketAccept
from .event_ps import EventProcessExec,EventProcessKill,EventProcessSetuid
from .event_sys import EventSysSethostname
from .event_tcp import EventTcpRecvRst,EventTcpSendRst
from .event_buffer import EventBuffer

predefined_events = [
    EventFileOpen,
    EventFileUnlink,
    EventFileTruncate,
    EventFileChown,
    EventFileChmod,
    EventFileMkdir,
    EventFileRmdir,
    EventFileSymlink,
    EventSocketListen,
    EventSocketConnect,
    EventSocketAccept,
    EventProcessExec,
    EventProcessKill,
    EventProcessSetuid,
    EventSysSethostname,
    EventTcpRecvRst,
    EventTcpSendRst
]

class EventManager(object):
    def __init__(self, pid_filter=[], uid_filter=[]):
        self.tp = TracePoint(pid_filter=pid_filter, uid_filter=uid_filter)
        self.syscall_hash = {}
        self.stats = {}
        self.callback = None
        self.event_objs = []
        self.init_events()
        self.evt_id = 0

    def init_events(self):
        for cls in predefined_events:
            event_obj = cls(self)
            self.event_objs.append(event_obj)

    def event_register(self, event_name):
        self.stats[event_name] = 0

    def event_send(self, args):
        name = args['name']
        if name in self.stats:
            self.stats[name] += 1
        else:
            self.stats[name] = 1
        if self.callback:
            args['id'] = self.evt_id
            self.callback(args)
        self.evt_id += 1
        
    def event_watch(self, event, callback):
        self.tp.add_event_watch(event, callback)

    def collect(self, callback):
        self.callback = callback
        self.tp.start()

    def file_read(self, event_file, callback):
        eb = EventBuffer(file_path=event_file, persist=True)
        event_nb = eb.length()
        for pos in range(0, event_nb):
            event = eb.read(pos)
            name = event['name']
            if name in self.stats:
                self.stats[name] += 1
            else:
                self.stats[name] = 1
            callback(event)
        return event_nb

    def stop(self):
        self.tp.stop()

    def get_stats(self):
        return self.stats

