#!/usr/bin/python3
# vim: set filetype=python
import os
import sys
import re
import json
import subprocess
import time

class EventProcessKill(object):
    def __init__(self, event_mgr):
        self.syscall_hash = {}
        self.event_name = "kill"
        self.em = event_mgr
        self.em.event_register(self.event_name)
        self.em.event_watch("syscalls/sys_enter_kill", self.syscalls_sys_enter_kill)
        self.em.event_watch("syscalls/sys_exit_kill", self.syscalls_sys_exit_kill)

    def syscalls_sys_enter_kill(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        self.syscall_hash[syscall_nr] = event

    def syscalls_sys_exit_kill(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        ret = event['parameters']['ret']
        if syscall_nr in self.syscall_hash:
            event = self.syscall_hash[syscall_nr]
            kill_pid = event['parameters']['pid']
            sig = event['parameters']['sig']
            detail = f"pid={kill_pid} signal={sig} ret={ret}"
            args = {
               "name":self.event_name,
               "comm": event["comm"],
               "pid": event["pid"],
               "uid": event["uid"],
               "detail": detail,
               "ok": True if ret >= 0 else False
            }
            del self.syscall_hash[syscall_nr]
            self.em.event_send(args)

class EventProcessExec(object):
    def __init__(self, event_mgr):
        self.syscall_hash = {}
        self.event_name = "exec"
        self.em = event_mgr
        self.em.event_register(self.event_name)
        self.em.event_watch("syscalls/sys_enter_execve", self.syscalls_sys_enter_execve)
        self.em.event_watch("syscalls/sys_exit_execve", self.syscalls_sys_exit_execve)

    def syscalls_sys_enter_execve(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        self.syscall_hash[syscall_nr] = event

    def syscalls_sys_exit_execve(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        ret = event['parameters']['ret']
        if syscall_nr in self.syscall_hash:
            event = self.syscall_hash[syscall_nr]
            args = {
               "name": self.event_name,
               "comm": event["comm"],
               "pid": event["pid"],
               "uid": event["uid"],
               "detail": " ".join(event['parameters']['argv']),
               "ok": True if ret == 0 else False
            }
            del self.syscall_hash[syscall_nr]
            self.em.event_send(args)

class EventProcessSetuid(object):
    def __init__(self, event_mgr):
        self.syscall_hash = {}
        self.event_name = "setuid"
        self.em = event_mgr
        self.em.event_register(self.event_name)
        self.em.event_watch("syscalls/sys_enter_setuid", self.syscalls_sys_enter_setuid)
        self.em.event_watch("syscalls/sys_exit_setuid", self.syscalls_sys_exit_setuid)

    def syscalls_sys_enter_setuid(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        self.syscall_hash[syscall_nr] = event

    def syscalls_sys_exit_setuid(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        ret = event['parameters']['ret']
        if syscall_nr in self.syscall_hash:
            event = self.syscall_hash[syscall_nr]
            set_uid = event['parameters']['uid']
            detail = f"uid={set_uid} ret={ret}"
            args = {
               "name":self.event_name,
               "comm": event["comm"],
               "pid": event["pid"],
               "uid": event["uid"],
               "detail": detail,
               "ok": True if ret >= 0 else False
            }
            del self.syscall_hash[syscall_nr]
            self.em.event_send(args)

