#!/usr/bin/python3
# vim: set filetype=python
import os
import sys
import re
import json
import subprocess
import time

class EventSocketListen(object):
    def __init__(self, event_mgr):
        self.syscall_hash = {}
        self.event_name = "listen"
        self.em = event_mgr
        self.em.event_register(self.event_name)
        self.em.event_watch("syscalls/sys_enter_listen", self.syscalls_sys_enter_listen)
        self.em.event_watch("syscalls/sys_exit_listen", self.syscalls_sys_exit_listen)

    def syscalls_sys_enter_listen(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        self.syscall_hash[syscall_nr] = event

    def syscalls_sys_exit_listen(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        ret = event['parameters']['ret']
        if syscall_nr in self.syscall_hash:
            event = self.syscall_hash[syscall_nr]
            fd = event['parameters']['fd']
            backlog = event['parameters']['backlog']
            detail = f"fd={fd} backlog={backlog} ret={ret}"
            args = {
               "name":self.event_name,
               "comm": event["comm"],
               "pid": event["pid"],
               "uid": event["uid"],
               "detail": detail,
               "ok": True if ret >= 0 else False
            }
            del self.syscall_hash[syscall_nr]
            self.em.event_send(args)

class EventSocketConnect(object):
    def __init__(self, event_mgr):
        self.syscall_hash = {}
        self.event_name = "connect"
        self.em = event_mgr
        self.em.event_register(self.event_name)
        self.em.event_watch("syscalls/sys_enter_connect", self.syscalls_sys_enter_connect)
        self.em.event_watch("syscalls/sys_exit_connect", self.syscalls_sys_exit_connect)

    def syscalls_sys_enter_connect(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        self.syscall_hash[syscall_nr] = event

    def syscalls_sys_exit_connect(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        ret = event['parameters']['ret']
        if syscall_nr in self.syscall_hash:
            event = self.syscall_hash[syscall_nr]
            if 'uservaddr' in event['parameters']:
                addr = event['parameters']['uservaddr']
            else:
                addr = ''
            detail = f"address={addr} ret={ret}"
            args = {
               "name":self.event_name,
               "comm": event["comm"],
               "pid": event["pid"],
               "uid": event["uid"],
               "detail": detail,
               "ok": True if ret >= 0 else False
            }
            del self.syscall_hash[syscall_nr]
            self.em.event_send(args)

class EventSocketAccept(object):
    def __init__(self, event_mgr):
        self.syscall_hash = {}
        self.event_name = "accept"
        self.em = event_mgr
        self.em.event_register(self.event_name)
        self.em.event_watch("syscalls/sys_enter_accept", self.syscalls_sys_enter_accept)
        self.em.event_watch("syscalls/sys_exit_accept", self.syscalls_sys_exit_accept)

    def syscalls_sys_enter_accept(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        self.syscall_hash[syscall_nr] = event

    def syscalls_sys_exit_accept(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        ret = event['parameters']['ret']
        if syscall_nr in self.syscall_hash:
            event = self.syscall_hash[syscall_nr]
            if 'upeer_sockaddr' in event['parameters']:
                addr = event['parameters']['upeer_sockaddr']
            else:
                addr = ''
            detail = f"address={addr} ret={ret}"
            args = {
               "name":self.event_name,
               "comm": event["comm"],
               "pid": event["pid"],
               "uid": event["uid"],
               "detail": detail,
               "ok": True if ret >= 0 else False
            }
            del self.syscall_hash[syscall_nr]
            self.em.event_send(args)

