#!/usr/bin/python3
# vim: set filetype=python
import os
import sys
import re
import json
import subprocess
import time

class EventSysSethostname(object):
    def __init__(self, event_mgr):
        self.syscall_hash = {}
        self.event_name = "sethostname"
        self.em = event_mgr
        self.em.event_register(self.event_name)
        self.em.event_watch("syscalls/sys_enter_sethostname", self.syscalls_sys_enter_sethostname)
        self.em.event_watch("syscalls/sys_exit_sethostname", self.syscalls_sys_exit_sethostname)

    def syscalls_sys_enter_sethostname(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        self.syscall_hash[syscall_nr] = event

    def syscalls_sys_exit_sethostname(self, event):
        syscall_nr = event['parameters']['__syscall_nr']
        ret = event['parameters']['ret']
        if syscall_nr in self.syscall_hash:
            event = self.syscall_hash[syscall_nr]
            name = event['parameters']['name']
            detail = f"hostname={name} ret={ret}"
            args = {
               "name":self.event_name,
               "comm": event["comm"],
               "pid": event["pid"],
               "uid": event["uid"],
               "detail": detail,
               "ok": True if ret >= 0 else False
            }
            del self.syscall_hash[syscall_nr]
            self.em.event_send(args)

