import os
import sys
import shlex
import re
import subprocess
from setuptools import setup,find_packages
from setuptools.command.install import install
from setuptools.command.build_ext import build_ext

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def build_install_libbpf():
    if os.path.exists('/usr/include/bpf/libbpf.h'):
        print("libbpf is already installed")
        return
    libbpf_dir_src=os.path.join(os.path.dirname(__file__), 'libbpf', 'src')
    cmd_str = "cd {} && make && make install".format(shlex.quote(libbpf_dir_src))
    ret = os.WEXITSTATUS(os.system(cmd_str))

def get_os_id():
    d = {}
    with open('/etc/os-release', 'r') as fd:
        os_release = fd.read()
    for line in os_release.splitlines():
        try:
            k,v = line.rstrip().split("=")
        except:
            continue
        # .strip('"') will remove if there or else do nothing
        d[k] = v.strip('"')
    return d['ID'].lower()

def install_clang():
    rc = subprocess.call(['which', 'clang'], stdout=subprocess.DEVNULL)
    if rc == 0:
        print("clang is already installed")
        return
    print("installing clang/llvm")
    os_id = get_os_id()
    if os_id == 'ubuntu':
        rc = subprocess.call(['apt-get', 'install', '-y', 'clang'], stdout=subprocess.DEVNULL)

def install_libjsonc():
    result=os.popen('whereis libjson-c.a').read()
    if re.search('libjson-c.a', result):
        print("libjson-c is already installed")
        return
    os_id = get_os_id()
    print("installing libjson-c")
    if os_id == 'ubuntu':
        rc = subprocess.call(['apt-get', 'install', '-y', 'libjson-c-dev'], stdout=subprocess.DEVNULL)

class CustomInstall(install):
    def run(self):
        install.run(self)
        build_install_libbpf()
        install_clang()
        install_libjsonc()

setup(
    name = "psevent",
    version = "1.0.3",
    author = "Fei Chen",
    author_email = "feiphilchen@gmail.com",
    description = ("Watch and filter process events with a curse window"),
    license = 'GNU General Public License version 3.0 (GPLv3)',
    keywords = "events,ebpf,trace",
    url = "https://github.com/feiphilchen/psevent",
    packages=['psevent', 'psevent/trace_event'],
    long_description=read('README.md'),
    scripts=['bin/psevent'],
    install_requires=[
        "click"
    ],
    zip_safe=False,
    include_package_data=True,
    package_data={
        "": ["*.c", "*.h"],
    },
    cmdclass={
        "install": CustomInstall
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ]
)
